const axios = require('axios');
const yaml = require('yaml');
const logger = require('../utils/logger');
const ClashParser = require('./clashParser');
const { Node } = require('../models');

class SubscriptionManager {
  constructor() {
    this.subscriptionUrl = process.env.CLASH_SUBSCRIPTION_URL;
    this.updateInterval = parseInt(process.env.SUBSCRIPTION_UPDATE_INTERVAL) || 3600000; // 默认1小时
  }

  /**
   * 从订阅地址获取配置
   */
  async fetchSubscription() {
    if (!this.subscriptionUrl) {
      throw new Error('未配置订阅地址');
    }

    try {
      // 简化日志输出

      const response = await axios.get(this.subscriptionUrl, {
        timeout: 30000,
        headers: {
          'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36'
        }
      });

      // 检查响应内容
      if (!response.data) {
        throw new Error('订阅地址返回空内容');
      }

      // 尝试解析为YAML
      let config;
      let rawContent = response.data;
      
      try {
        config = yaml.parse(rawContent);
      } catch (error) {
        // 如果不是YAML，尝试Base64解码
        try {
          const decoded = Buffer.from(rawContent, 'base64').toString('utf8');
          config = yaml.parse(decoded);
        } catch (decodeError) {
          // 如果Base64解码也失败，尝试直接解析原始内容
          // 有些订阅可能包含多个Base64编码的配置，用换行符分隔
          const lines = rawContent.split('\n').filter(line => line.trim());
          let decodedContent = '';
          
          for (const line of lines) {
            try {
              const decoded = Buffer.from(line, 'base64').toString('utf8');
              decodedContent += decoded + '\n';
            } catch (e) {
              // 如果某行不是Base64，跳过
              continue;
            }
          }
          
          if (decodedContent) {
            config = yaml.parse(decodedContent);
          } else {
          throw new Error('无法解析订阅内容，既不是有效的YAML也不是Base64编码');
          }
        }
      }

      if (!config.proxies || !Array.isArray(config.proxies)) {
        throw new Error('订阅配置中没有找到有效的代理节点');
      }

      // 简化日志输出

      return config;
    } catch (error) {
      logger.error('获取订阅配置失败', { 
        error: error.message, 
        url: this.subscriptionUrl 
      });
      throw error;
    }
  }

  /**
   * 更新节点列表
   */
  async updateNodes() {
    try {
      // 简化日志输出

      // 获取订阅配置
      const config = await this.fetchSubscription();
      
      // 解析节点
      const parser = new ClashParser();
      const newNodes = [];
      
      for (const proxy of config.proxies) {
        const node = parser.parseProxy(proxy);
        if (node) {
          newNodes.push(node);
        }
      }

      if (newNodes.length === 0) {
        logger.warn('订阅中没有找到有效的节点');
        return { updated: 0, added: 0, removed: 0 };
      }

      // 获取现有节点
      const existingNodes = await Node.findAll({
        where: { isActive: true }
      });

      const existingNodeMap = new Map();
      existingNodes.forEach(node => {
        const key = `${node.name}-${node.server}-${node.port}`;
        existingNodeMap.set(key, node);
      });

      let added = 0;
      let updated = 0;
      let removed = 0;

      // 处理新节点
      for (const nodeData of newNodes) {
        const key = `${nodeData.name}-${nodeData.server}-${nodeData.port}`;
        const existingNode = existingNodeMap.get(key);

        if (existingNode) {
          // 更新现有节点
          await existingNode.update({
            ...nodeData,
            updatedAt: new Date()
          });
          updated++;
          existingNodeMap.delete(key);
        } else {
          // 添加新节点
          await Node.create({
            ...nodeData,
            isActive: true,
            status: 'offline'
          });
          added++;
        }
      }

      // 标记不再存在的节点为非活跃
      for (const [key, node] of existingNodeMap) {
        await node.update({ isActive: false });
        removed++;
      }

      logger.info(`节点列表更新完成 - 新增${added}个，更新${updated}个，移除${removed}个`);

      return { updated: newNodes.length, added, updated, removed };
    } catch (error) {
      logger.error('更新节点列表失败', { error: error.message });
      throw error;
    }
  }

  /**
   * 启动自动更新
   */
  startAutoUpdate() {
    if (!this.subscriptionUrl) {
      logger.warn('未配置订阅地址，跳过自动更新');
      return;
    }

    logger.info('启动订阅自动更新', { 
      interval: `${this.updateInterval / 1000}秒` 
    });

    // 立即执行一次更新
    this.updateNodes().catch(error => {
      logger.error('初始订阅更新失败', { error: error.message });
    });

    // 设置定时更新
    this.updateTimer = setInterval(() => {
      this.updateNodes().catch(error => {
        logger.error('定时订阅更新失败', { error: error.message });
      });
    }, this.updateInterval);
  }

  /**
   * 停止自动更新
   */
  stopAutoUpdate() {
    if (this.updateTimer) {
      clearInterval(this.updateTimer);
      this.updateTimer = null;
      logger.info('订阅自动更新已停止');
    }
  }

  /**
   * 手动更新
   */
  async manualUpdate() {
    try {
      const result = await this.updateNodes();
      return {
        success: true,
        data: result
      };
    } catch (error) {
      return {
        success: false,
        error: error.message
      };
    }
  }

  /**
   * 获取订阅状态
   */
  getStatus() {
    return {
      subscriptionUrl: this.subscriptionUrl,
      updateInterval: this.updateInterval,
      autoUpdateEnabled: !!this.updateTimer,
      lastUpdate: this.lastUpdateTime
    };
  }
}

module.exports = SubscriptionManager; 